@@ -18,39 +18,43 @@
 #include <assert.h>
 #include "NodeList.h"
 #include <algorithm>
+#include <cstring>
+#include <typeinfo>
+
 using namespace std;
+
 enum nodeType {UNDEF, IN, OUT, BOOLEAN, MUTANT, ANALPTS, DELAY, SUSTAIN, RUNTIME, NOISE, COMMENT};//rework input parameters
 char line[1000000000];
 int offset = 100000000;
 
-	/**
-	 * Default Constructor
-	 */
+/**
+ * Default Constructor
+ */
 NodeItem::NodeItem() : node(NULL)
 {
 }
 
-	/**
-	 * copy constructor
-	 */
+/**
+ * copy constructor
+ */
 NodeItem::NodeItem(const NodeItem & other) : node(NULL)
 {
 	//use operator = to assign the data in "other" to this NodeItem
 	*this = other;
 }
 
-	/**
-	 * Destructor
-	 */
+/**
+ * Destructor
+ */
 NodeItem::~NodeItem()
 {
 	//deallocate memory used to store a node
 	removeNode();
 }
 
-	/**
-	 * assign one NodeItem to another
-	 */
+/**
+ * assign one NodeItem to another
+ */
 NodeItem & NodeItem::operator = (const NodeItem & other)
 {
 	//if they are not the same object
@@ -72,9 +76,9 @@ NodeItem & NodeItem::operator = (const NodeItem & other)
 	return *this;
 }
 
-	/**
-	 * set the node on this item to a valid InputNode, OutputNode, BooleanNode, DelayNode, or SustainNode
-	 */
+/**
+ * set the node on this item to a valid InputNode, OutputNode, BooleanNode, DelayNode, or SustainNode
+ */
 void NodeItem::setNode(const Node & other)
 {
 	//use RTTI to call the proper function
@@ -100,9 +104,9 @@ void NodeItem::setNode(const Node & other)
 	}
 }
 
-	/**
-	 * set the node to a valid InputNode
-	 */
+/**
+ * set the node to a valid InputNode
+ */
 void NodeItem::setNode(const InputNode & other)
 {
 	//deallocate old memory
@@ -124,9 +128,9 @@ void NodeItem::setNode(const InputNode & other)
 	}
 }
 
-	/**
-	 * set the node to a valid OutputNode
-	 */
+/**
+ * set the node to a valid OutputNode
+ */
 void NodeItem::setNode(const OutputNode & other)
 {
 	//deallocate old memory
@@ -148,9 +152,9 @@ void NodeItem::setNode(const OutputNode & other)
 	}
 }
 
-	/**
-	 * set the node to a valid BooleanNode
-	 */
+/**
+ * set the node to a valid BooleanNode
+ */
 void NodeItem::setNode(const BooleanNode & other)
 {
 	//deallocate old memory
@@ -172,9 +176,9 @@ void NodeItem::setNode(const BooleanNode & other)
 	}
 }
 
-	/**
-	 * set the node to a valid DelayNode
-	 */
+/**
+ * set the node to a valid DelayNode
+ */
 void NodeItem::setNode(const DelayNode & other)
 {
 	//deallocate old memory
@@ -196,9 +200,9 @@ void NodeItem::setNode(const DelayNode & other)
 	}
 }
 
-	/**
-	 * set the node to a valid SustainNode
-	 */
+/**
+ * set the node to a valid SustainNode
+ */
 void NodeItem::setNode(const SustainNode & other)
 {
 	//deallocate old memory
@@ -220,9 +224,9 @@ void NodeItem::setNode(const SustainNode & other)
 	}
 }
 
-	/**
-	 * check if a node has been allocated to this NodeItem
-	 */
+/**
+ * check if a node has been allocated to this NodeItem
+ */
 bool NodeItem::hasNode() const
 {
 	//check for NULL
@@ -236,9 +240,9 @@ bool NodeItem::hasNode() const
 	}
 }
 
-	/**
-	 * get the non-NULL node from the NodeItem
-	 */
+/**
+ * get the non-NULL node from the NodeItem
+ */
 Node &NodeItem::getNode()
 {
 	//throw an exception if the user is trying to derefrence NULL
@@ -250,9 +254,9 @@ Node &NodeItem::getNode()
 	return *node;
 }
 
-	/**
-	 * remove a node by reclaiming its memory
-	 */
+/**
+ * remove a node by reclaiming its memory
+ */
 bool NodeItem::removeNode()
 {
 	//only call delete if pointer is non-NULL
@@ -265,73 +269,74 @@ bool NodeItem::removeNode()
 	return false;
 }
 
-	/**
-	 * change the name used to refer to this NodeItem
-	 */
+/**
+ * change the name used to refer to this NodeItem
+ */
 void NodeItem::setName(string s)
 {
 	name = s;
 }
 
-	/**
-	 * Constructor
-	 */
+/**
+ * Constructor
+ */
 NodeList::NodeList()
 {
 	runTime = 100;
 }
 
-	/**
-	 * Adds item to outputs
-	 */
+/**
+ * Adds item to outputs
+ */
 void NodeList::addOutput(NodeItem &item)
 {
 	outputs.push_back(item);
 }
 
-	/**
-	 * Adds item to inputs
-	 */
+/**
+ * Adds item to inputs
+ */
 void NodeList::addInput(NodeItem &item)
 {
 	inputs.push_back( item );
 }
 
-	/**
-	 * Adds item to outCalc
-	 */
+/**
+ * Adds item to outCalc
+ */
 void NodeList::addOutCalc(NodeItem &item)
 {
 	outCalc.push_back(item);
 }
 
-	/**
-	 * adds item to nodes
-	 */
+/**
+ * adds item to nodes
+ */
 void NodeList::addNode(NodeItem &item)
 {
 	nodes.push_back(item);
 }
 
-	/**
-	 * Adds item to mutants
-	 */
+/**
+ * Adds item to mutants
+ */
 void NodeList::addMutant(Mutant &item)
 {
 	mutants.push_back(item);
 }
 
-	/**
-	 * Adds item to analPts
-	 */
+/**
+ * Adds item to analPts
+ */
 void NodeList::addAnalPt( const int &item )
 {
 	analPts.push_back(item);
 }
 
-	/**
-	 * Comment to be written
-	 */
+/**
+ * For every Mutant, find the matching node and set its Mut
+ * to the matching Mutant's type
+ */
 void NodeList::setMutants( void )
 {
 	if( mutants.size() > 0 )  //If any mutants, then update the nodes
@@ -350,9 +355,9 @@ void NodeList::setMutants( void )
 	} 
 }
 
-	/**
-	 * Read a line from the working buffer
-	 */
+/**
+ * Read a line from the working buffer
+ */
 void readLine(istream &in)
 {
 	in.getline(line, sizeof(line));
@@ -362,27 +367,27 @@ void readLine(istream &in)
 }
 
 
-	/**
-	 * Test if line is exhausted
-	 */
+/**
+ * Test if line is exhausted
+ */
 bool isEmpty()
 {
 	if (int(offset) < int(sizeof(line)) && line[offset] != '\0') return false;
 	return true;
 }
 
-	/**
-	 * Test if input is exhausted
-	 */
+/**
+ * Test if input is exhausted
+ */
 bool isEOF(istream &in)
 {
 	if (isEmpty() && in.eof()) return true;
 	return false;
 }
 
-	/**
-	 * Read a character from the working buffer
-	 */
+/**
+ * Read a character from the working buffer
+ */
 char readChar(istream &in)
 {
 	if (isEmpty()) {
@@ -392,9 +397,9 @@ char readChar(istream &in)
 	return line[offset++];
 }
 
-	/**
-	 * Read in a single word, deliminated by either ':', ",", or "\n"
-	 */
+/**
+ * Read in a single word, deliminated by either ':', ",", or "\n"
+ */
 string readWord(istream &in, char &terminator)
 {
 	string s;
@@ -420,17 +425,17 @@ string readWord(istream &in, char &terminator)
 	return s;
 }
 
-	/**
-	 *  The remainder of the input line is read and discarded.
-	 */
+/**
+ *  The remainder of the input line is read and discarded.
+ */
 void skipLine(istream &in)
 {
 	offset = sizeof(line);
 }
 
-	/**
-	 * Read in the line type
-	 */
+/**
+ * Read in the line type
+ */
 nodeType readType(istream &in)
 {
 	char t;
@@ -469,9 +474,9 @@ nodeType readType(istream &in)
 	} else return UNDEF;
 }
 
-	/**
-	 * Read in a chemical node name and return the name
-	 */
+/**
+ * Read in a chemical node name and return the name
+ */
 string readName(istream &in)
 {
 	char t;
@@ -479,9 +484,9 @@ string readName(istream &in)
 	return s;
 }
 
-	/**
-	 * Read in an integer value
-	 */
+/**
+ * Read in an integer value
+ */
 int readValue(istream &in, char &t)
 {
 	char *tptr;
@@ -491,9 +496,9 @@ int readValue(istream &in, char &t)
 	return static_cast<int>(i);
 }
 
-	/**
-	 * Read in a float value
-	 */
+/**
+ * Read in a float value
+ */
 float readFloat( istream &in )
 {
 	char t;//, *tptr;
@@ -503,9 +508,9 @@ float readFloat( istream &in )
 	return static_cast<float>(i);
 }
 
-	/**
-	 * Read the intial value for a chemical node: either True or False
-	 */
+/**
+ * Read the intial value for a chemical node: either True or False
+ */
 bool readInitial(string & src, string & inputStr )
 {
 	ifstream in;
@@ -526,21 +531,25 @@ bool readInitial(string & src, string & inputStr )
 	return true;
 }
 
-	/** 
-	 * Read the intial value for a chemical node: either True or False
-	 */
+/** 
+ * Read the intial value for a chemical node: either True or False
+ */
 bool readInitial(string & init )
 {
-	if (toupper(init[0]) == 'F'){
-		//cerr << "Init: False" << endl;
-		return false;
-	}
-	return true;
+	bool rval;
+	if (toupper(init[0]) == 'F')
+		rval = false;
+	else if (toupper(init[0]) == 'T')
+		rval = true;
+	else
+		rval = rand()%2;
+
+	return rval;
 }
 
-	/** 
-	 * Read the intial value for a chemical node: either True or False
-	 */
+/** 
+ * Read the intial value for a chemical node: either True or False
+ */
 string readInitialSrc(istream & in )
 {
 	char t;
@@ -548,9 +557,9 @@ string readInitialSrc(istream & in )
 	return s;
 }
 
-	/** 
-	 * Read in a chemical node name, locate and return its index
-	 */
+/** 
+ * Read in a chemical node name, locate and return its index
+ */
 int readSource(istream &in, NodeList &nl, int &source)
 {
 	char t;
@@ -560,9 +569,10 @@ int readSource(istream &in, NodeList &nl, int &source)
 	return false;
 }
 
-	/**
-	 * Comment to be written
-	 */
+/**
+ * read a word into string & mutant and return true if
+ * istream is empty
+ */
 bool readMutant(istream &in, NodeList &nl, string & mutant)
 {
 	char t;
@@ -572,9 +582,9 @@ bool readMutant(istream &in, NodeList &nl, string & mutant)
 	return false;
 }
 
-	/** 
-	 * Read a decision table: a string of T and F characters
-	 */
+/** 
+ * Read a decision table: a string of T and F characters
+ */
 int readDecision(istream &in, int &decision)
 {
 	char t;
@@ -589,9 +599,9 @@ int readDecision(istream &in, int &decision)
 	return false;
 }
 
-	/**
-	 * Comment to be written, function not in use
-	 */
+/**
+ * Comment to be written, function not in use
+ */
 void setRandInputs( InputNode & n, NodeList &nl )  //This function is not being used
 {
 	int dosage = rand()%101;
@@ -602,9 +612,9 @@ void setRandInputs( InputNode & n, NodeList &nl )  //This function is not being
 	n.setDuration(nl.getRunTime()-1);
 }
 
-	/**
-	 * Read points of output analysis
-	 */
+/**
+ * Read points of output analysis
+ */
 void readAnalPts( istream &in, NodeList & nl )
 {
 	char t;
@@ -657,10 +667,10 @@ void readAnalPts( istream &in, NodeList & nl )
 		cerr << "\n Added " << nl.ptsSize() << " output analysis points." << endl;
 }
 
-	/**
-	 * Reads the start, dosage, and duration from
-	 * the instream for an InputNode (Tomas Helikar 9/26/05)
-	 */
+/**
+ * Reads the start, dosage, and duration from
+ * the instream for an InputNode (Tomas Helikar 9/26/05)
+ */
 bool readParams(istream &in, InputNode & n, bool random, NodeList & nl )
 {
 	int start, dos, dur;
@@ -732,10 +742,10 @@ bool readParams(istream &in, InputNode & n, bool random, NodeList & nl )
 	return rval;
 }
 
-	/**
-	 * Reads in an Inputfile and sets up
-	 * the network based on that file
-	 */
+/**
+ * Reads in an Inputfile and sets up
+ * the network based on that file
+ */
 istream& operator >>(istream &in, NodeList &nl)
 {
 	while (!isEOF(in)) {
@@ -760,267 +770,305 @@ istream& operator >>(istream &in, NodeList &nl)
 
 				} break;
 			case IN: {
-				
-					 string name;
-					 name = readName(in);
-					 if (verbose) 
-						 cerr << "Input Node: " << name << endl;
-					 item.setName(name);
-					 bool random = false;
 
-					 InputNode n;
-					 string src = readInitialSrc( in );
-					 if( src == "File" )//input handling should be done elsewhere
-					 {
-						 int dosage = 0;
-						 n.setNoise(false);
-						 n.setManual(true);
-						 string inputStr = "";
-						 char t;
-						 string inSrc = readWord(in,t);
-						 if( readWord( in,t ) == "R") 
+						 string name;
+						 name = readName(in);
+						 if (verbose) 
+							 cerr << "Input Node: " << name << endl;
+						 item.setName(name);
+						 bool random = false;
+
+						 InputNode n;
+						 string src = readInitialSrc( in );
+						 if( src == "File" )//input handling should be done elsewhere
 						 {
-							 //if( rinputs )
-								 random = true;
-							 string sd = readWord( in, t );
-							 StringUtils su;
-							 vector<string> rangeStr;
-							 int cnt;
-							 cnt = su.SplitString(sd,"-",rangeStr);
-							 if( rangeStr.size() == 2 )
+							 int dosage = 0;
+							 n.setNoise(false);
+							 n.setManual(true);
+							 string inputStr = "";
+							 char t;
+							 string inSrc = readWord(in,t);
+							 if( readWord( in,t ) == "R") 
 							 {
-								 int dos;
-								 int low = atoi(rangeStr[0].c_str());
-								 int high = atoi(rangeStr[1].c_str());
-								 dos = rand()%(high-low+1)+low;
-								 if( verbose )
-									 cout << "Percent rand: " << dos << endl;
-								 char dosStr [10];
-								 sprintf( dosStr,"%d", dos );
-								 string tmpStr = "stats/" + nl.getRunName() + "/";
-								 inSrc = tmpStr + inSrc;
-								 inSrc += dosStr;
-								 inSrc += ".txt";
-								 dosage = dos;
-								 cerr << "Src: " << inSrc << endl;
+								 //if( rinputs )
+								 random = true;
+								 string sd = readWord( in, t );
+								 StringUtils su;
+								 vector<string> rangeStr;
+								 int cnt;
+								 cnt = su.SplitString(sd,"-",rangeStr);
+								 if( rangeStr.size() == 2 )
+								 {
+									 int dos;
+									 int low = atoi(rangeStr[0].c_str());
+									 int high = atoi(rangeStr[1].c_str());
+									 dos = rand()%(high-low+1)+low;
+									 if( verbose )
+										 cout << "Percent rand: " << dos << endl;
+									 char dosStr [10];
+									 sprintf( dosStr,"%d", dos );
+									 string tmpStr = "stats/" + nl.getRunName() + "/";
+									 inSrc = tmpStr + inSrc;
+									 inSrc += dosStr;
+									 inSrc += ".txt";
+									 dosage = dos;
+									 cerr << "Src: " << inSrc << endl;
+								 }
+								 else
+								 {
+									 cerr << "Wrong format random number range in your specs file!" 
+										 << endl;
+									 exit(1);
+								 }
+
 							 }
 							 else
 							 {
-								 cerr << "Wrong format random number range in your specs file!" 
+								 dosage = -1;
+								 //n.setDuration(nl.getRunTime()-1);
+							 }
+							 n.setValue(readInitial( inSrc, inputStr ));
+							 n.setInputBitString(inputStr, nl);
+							 n.setDosage(dosage);
+							 nl.dosageRec.push_back(dosage);
+						 }
+						 else
+						 {
+							 n.setManual(false);
+							 //string src = readInitialSrc( in );
+							 //		 string s= "False";
+							 //		 n.setValue(readInitial(s));
+
+							 char t;
+							 //Read in if the dosage is random or fixed
+							 string tmp = readWord( in, t );
+
+							 // cout << "Read: " << tmp << endl;
+							 if( strcmp(tmp.c_str(), "R" ) == 0 )
+							 {
+								 //if( rinputs )
+								 random = true;
+							 }
+							 //Check for fixed or random dosages
+							 else if( strcmp(tmp.c_str(), "F" ) == 0 )
+								 random = false;
+							 else
+							 {
+								 cerr << "Have to specify if the input is fixed or random" 
 									 << endl;
 								 exit(1);
 							 }
 
-						 }
-						 else
-						 {
-							 dosage = -1;
-							 //n.setDuration(nl.getRunTime()-1);
-						 }
-						 n.setValue(readInitial( inSrc, inputStr ));
-						 n.setInputBitString(inputStr, nl);
-						 n.setDosage(dosage);
-						 nl.dosageRec.push_back(dosage);
-					 }
-					 else
-					 {
-						 n.setManual(false);
-						 //string src = readInitialSrc( in );
-				//		 string s= "False";
-				//		 n.setValue(readInitial(s));
+							 tmp = readWord( in, t );
 
-						 char t;
-						 //Read in if the dosage is random or fixed
-						 string tmp = readWord( in, t );
-						 // cout << "Read: " << tmp << endl;
-						 if( strcmp(tmp.c_str(), "R" ) == 0 )
-						 {
-							 //if( rinputs )
-								 random = true;
-						 }
-						 //Check for fixed or random dosages
-						 else if( strcmp(tmp.c_str(), "F" ) == 0 )
-							 random = false;
-						 else
-						 {
-							 cerr << "Have to specify if the input is fixed or random" 
-								 << endl;
-							 exit(1);
-						 }
-
-						 tmp = readWord( in, t );
-
-						 //Check for noise 
-						 if( strcmp(tmp.c_str(), "N" ) == 0 )
-						 {
-							 if( nl.isNoisyInputs() )
+							 //Check for noise 
+							 if( strcmp(tmp.c_str(), "N" ) == 0 )
 							 {
-								 n.setNoise(true);
+								 if( nl.isNoisyInputs() )
+								 {
+									 n.setNoise(true);
+								 }
+							 }
+							 else if( strcmp(tmp.c_str(), "F" ) == 0 )
+							 {
+								 n.setNoise(false);
+							 }
+							 else
+							 {
+								 cerr << "Have to specify if the input is fixed or noisy" << endl;
+								 exit(1);
 							 }
-						 }
-						 else if( strcmp(tmp.c_str(), "F" ) == 0 )
-						 {
-							 n.setNoise(false);
-						 }
-						 else
-						 {
-							 cerr << "Have to specify if the input is fixed or noisy" << endl;
-							 exit(1);
-						 }
 
-						 bool done;                       
+							 bool done;                       
 
-						 //Ability to have different dosages throughout the simulation 
-						 //This section modified by Tomas Helikar 9/26/05
-						 do 
-						 {                               
-							 done = readParams(in, n, random, nl);
-							 n.checkDosages();
-						 } while(!done);
-						 n.reverseParams();
-						 //}
-						 n.setInjectionValue();
-						 
-						 n.setValue(readInitial(src));
-				 }
-				 item.setNode(n);
-				 nl.add(item);    // Add the node to the end of the list
-				 nl.addInput(item);
-				 if(calc)
-				 {
-					 nl.initInputBits();
-				 }
+							 //Ability to have different dosages throughout the simulation 
+							 //This section modified by Tomas Helikar 9/26/05
+							 do 
+							 {                               
+								 done = readParams(in, n, random, nl);
+								 n.checkDosages();
+							 } while(!done);
+							 n.reverseParams();
+							 //}
+							 n.setInjectionValue();
+
+							 n.setValue(readInitial(src));
+					 }
+					 item.setNode(n);
+					 nl.add(item);    // Add the node to the end of the list
+					 nl.addInput(item);
+					 if(calc)
+					 {
+						 nl.initInputBits();
+					 }
 		} break;
 		case OUT:{
-			int s;
-			if (verbose) cerr << "Output Node: ";
-			item.setName(readName(in));
-			OutputNode n;
-			string src = readInitialSrc( in );
-			n.setValue(readInitial(src));
-			//n.setValue(readInitial(in));
-			readSource(in, nl, s);
-			n.setSource(s);
-			item.setNode(n);
-			nl.addOutput(item);
-			nl.add(item);    // Add the node to the end of the lisit
-		} 
-		break;
+					 if(!outputAll)
+					 {
+						 int s;
+						 if (verbose) cerr << "Output Node: ";
+						 item.setName(readName(in));
+						 OutputNode n;
+						 string src = readInitialSrc( in );
+						 n.setValue(readInitial(src));
+						 //n.setValue(readInitial(in));
+						 readSource(in, nl, s);
+						 n.setSource(s);
+						 item.setNode(n);
+						 nl.addOutput(item);
+						 nl.add(item);    // Add the node to the end of the lisit
+					 }
+				 } 
+				 break;
 		case BOOLEAN: {
-				      if (verbose) cerr << "Boolean Node: ";
-				      item.setName(readName(in));
-				      BooleanNode n;
-				      string src = readInitialSrc( in );
-				      //n.setValue(readInitial(src));
-				      bool init = readInitial(src);
-				      n.setValue(init);
-				      bool done;
-				      do {
-					      int s;
-					      done = readSource(in, nl, s);
-					      n.addSource(s);
-				      } while(!done);
-				      do {
-					      int d;
-					      done = readDecision(in, d);
-					      n.addDecision(d);
-				      } while(!done);
-				      n.setTarget(!n.getValue());
-				      item.setNode(n);
-				      nl.add(item);          // Add the node to the end of the list
-				      if(calc) 
-				      {
-					      nl.addOutCalc(item);   
-					      // Gong added in 11/11/05 to add node into the net
-					      nl.avg.push_back(0);   
-					      // Gong added in 11/11/05 to initialize the Average vector
-					      nl.initBits(init);
-				      }
-			      } break;
-	case MUTANT:{
-			    string mutation;
-			    char t;
-			    mutation = readWord(in, t);
+						  if (verbose) cerr << "Boolean Node: ";
+						  string name;
+						  name = readName(in);
+						  item.setName(name);
+						  BooleanNode n;
+						  string src = readInitialSrc( in );
+						  //n.setValue(readInitial(src));
+						  bool init = readInitial(src);
+						  n.setValue(init);
+						  bool done;
+						  do {
+							  int s;
+							  done = readSource(in, nl, s);
+							  n.addSource(s);
+						  } while(!done);
+						  do {
+							  int d;
+							  done = readDecision(in, d);
+							  n.addDecision(d);
+						  } while(!done);
+						  n.setTarget(!n.getValue());
+						  item.setNode(n);
+						  nl.add(item);          // Add the node to the end of the list
+						  if(calc) 
+						  {
+							  nl.addOutCalc(item);   
+							  // Gong added in 11/11/05 to add node into the net
+							  nl.avg.push_back(0);   
+							  // Gong added in 11/11/05 to initialize the Average vector
+							  nl.initBits(init);
+						  }
 
-			    bool done = false;
-			    do {
-				    string name;
-				    done = readMutant(in, nl, name);
-				    Mutant mut(name, mutation);
-				    nl.addMutant(mut);
-			    } while(!done);
-		    } break;
-	case DELAY: {
-			    int s;
-			    char t;
-			    if (verbose) cerr << "Delay Node: ";
-			    item.setName(readName(in));
-			    DelayNode n;
-			    string src = readInitialSrc( in );
-			    n.setValue(readInitial(src));
-			    //n.setValue(readInitial(in));
-			    readSource(in, nl, s);
-			    n.setSource(s);
-			    n.setDelay(readValue(in, t));
-			    item.setNode(n);
-			    nl.add(item);    // Add the node to the end of the list
-		    } break;
-	case SUSTAIN: {
-			      int s;
-			      char t;
-			      if (verbose) cerr << "Sustained Node: ";
-			      item.setName(readName(in));
-			      SustainNode n;
-			      string src = readInitialSrc( in );
-			      n.setValue(readInitial(src));
-			      //n.setValue(readInitial(in));
-			      readSource(in, nl, s);
-			      n.setSource(s);
-			      n.setDuration(readValue(in, t));
-			      item.setNode(n);
-			      nl.add(item);    // Add the node to the end of the list
-			     
-		      } break;
-	case RUNTIME: 
-		      {
-			      char t;
-			      nl.setTransient(readValue(in, t));
-			      nl.setRunTime(readValue(in, t));
-			      break;
-		      }
-	case NOISE:
-		      {
-			      nl.setNoisyInputs();
-			      char t;
-			      nl.setWindow (readValue(in, t));
-			      nl.setScalar (readValue(in, t));
-			      break;
-		      }
-	case COMMENT:
-		      // Comments are ignored
-		      break;
-	case UNDEF:
-		      
-		      //use function to exit and output that input file is incorrect
-			      break;
-}
+						  //if outputall, create an output for every boolean.
+						  if(outputAll)
+						  {
+							  OutputNode o;
+							  NodeItem i;
+							  string oname="Out_"+name;
+							  i.setName(oname);
+							  o.setSource(nl.find(name));			  
+							  o.setValue(init);
+							  i.setNode(o);
+							  nl.addOutput(i);
+							  nl.add(i);
+						  }
+					  } break;
+		case MUTANT:{
+						string mutation;
+						char t;
+						mutation = readWord(in, t);
+
+						bool done = false;
+						do {
+							string name;
+							done = readMutant(in, nl, name);
+							Mutant mut(name, mutation);
+							nl.addMutant(mut);
+						} while(!done);
+					} break;
+		case DELAY: {
+						int s;
+						char t;
+						if (verbose) cerr << "Delay Node: ";
+						item.setName(readName(in));
+						DelayNode n;
+						string src = readInitialSrc( in );
+						n.setValue(readInitial(src));
+						//n.setValue(readInitial(in));
+						readSource(in, nl, s);
+						n.setSource(s);
+						n.setDelay(readValue(in, t));
+						item.setNode(n);
+						nl.add(item);    // Add the node to the end of the list
+					} break;
+		case SUSTAIN: {/*
+						  int s;
+						  char t;
+						  if (verbose) cerr << "Sustained Node: ";
+						  item.setName(readName(in));
+						  cout<<"Adding Sustain node:";
+						  SustainNode n;
+						  string src = readInitialSrc( in );
+						  cout<<"Source:"<<src<<" ";
+						  n.setValue(readInitial(src));
+						  //n.setValue(readInitial(in));
+						  readSource(in, nl, s);
+						  cout<<"Source:"<<s<<" ";
+						  n.setSource(s);
+						  int temp = readValue(in, t);
+						  //n.setDuration(readValue(in, t));
+						  cout<<"Duration:"<<temp<<" ";
+						  n.setDuration(temp);
+						  item.setNode(n);
+						  cout<<"Name:"<<item.getName()<<endl;
+						  nl.add(item);    // Add the node to the end of the list
+						  */
+						  char t=':';
+						  item.setName(readWord(in,t));
+						  SustainNode n;
+						  string temp = readWord(in,t);
+						  n.setValue(readInitial(temp));
+						  temp = readWord(in,t);
+						  n.setSource(nl.find(temp));
+						  n.setDuration(readValue(in,t));
+						  item.setNode(n);
+						  nl.add(item);
+
+					  } break;
+		case RUNTIME: 
+					  {
+						  char t;
+						  nl.setTransient(readValue(in, t));
+						  nl.setRunTime(readValue(in, t));
+						  break;
+					  }
+		case NOISE:
+					  {
+						  nl.setNoisyInputs();
+						  char t;
+						  nl.setWindow (readValue(in, t));
+						  nl.setScalar (readValue(in, t));
+						  break;
+					  }
+		case COMMENT:
+					  // Comments are ignored
+					  break;
+		case UNDEF:
+
+					  //use function to exit and output that input file is incorrect
+					  break;
+	}
 }
 return in;
 }
 
-	/**
-	 * Prints Analysis Points for output
-	 */
+/**
+ * Prints Analysis Points for output
+ */
 void NodeList::printAnalPts( void )
 {
 	for( int i = 0; i < int(analPts.size()); i++ )
 		cerr << "Point: " << analPts[i] << endl;
 }
 
-	/**
-	 *  Locate a given node by name. If it does not exist, create a reference
-	 *  to be filled in later.
-	 */
+/**
+ *  Locate a given node by name. If it does not exist, create a NodeItem
+ *  to be filled in later.
+ */
 int NodeList::find(string n)
 {
 	for (int i = 0; i < nodeSize(); i++) {
@@ -1036,15 +1084,17 @@ int NodeList::find(string n)
 	return nodeSize()-1;
 }
 
-	/**
-	 * Comment to be written
-	 */
-int NodeList::inOutCalc(string n)
-{
+/**
+ * Returns the index of the node with name n on the Outcalc array
+ * if that node is on the outcalc array, otherwise it returns -1
+ */
+int NodeList::inOutCalc(int i)
+{/*
 	int pos = -1;
 	bool rval = false;
 	int j = outCalcSize();
-	for (int i = 0; (i < j)&&(!rval); i++) {
+	for (int i = 0; (i < j)&&(!rval); i++) 
+	{
 		if (outCalc[i].getName() == n) 
 		{
 			if (verbose) cerr<<" Reference to node "<<n<<" found for average calculation "<<i<<endl;
@@ -1053,11 +1103,31 @@ int NodeList::inOutCalc(string n)
 		}
 	}
 	return pos;
+*/
+	return inOutIndex[i];
 }
 
-	/**
-	 * Returns the index number of node n
-	 */
+/**
+ *	initializes the in out array
+ */
+void NodeList::inOutInit(void)
+{
+	inOutIndex.resize(nodeSize(), -1);
+	for (int i = 0; i<nodeSize(); i++) 
+	{
+		for(int j = 0; j<outCalcSize(); ++j)
+		{
+			if (outCalc[j].getName() == getName(i))
+				inOutIndex[i] = j;
+		}
+	}
+	
+}
+
+
+/**
+ * Returns the index number of node n
+ */
 int NodeList::getNodeIndex(string n)
 {
 	int pos = -1;
@@ -1076,10 +1146,10 @@ int NodeList::getNodeIndex(string n)
 	return pos;
 }
 
-	/** 
-	 * Locate a given node by name. If it exists, replace the node reference.
-	 * Otherwise, add the item to the list.
-	 */
+/** 
+ * Locate a given node by name. If it exists, replace the node reference.
+ * Otherwise, add the item to the list.
+ */
 int NodeList::add(NodeItem &item)
 {
 	for (int i = 0; i < nodeSize(); i++) {
@@ -1098,24 +1168,37 @@ int NodeList::add(NodeItem &item)
 	return nodeSize()-1;
 }
 
-	/**
-	 * Prints required nodes to visout.
-	 */
+/**
+ * Prints required nodes to visout.
+ */
 void NodeList::visPrint(ostream &out, NodeList &nl)
 {
-	if (outputAll) {
-		for (int i = 0; i < nl.nodeSize(); i++) {
-			if (nl.getNode(i).getValue()) {
-				out << "* ";
-			} else {
-				out << ". ";
+	if (outputAll) 
+	{
+		for (int i = 0; i < nl.nodeSize(); i++) 
+		{
+			if(nl.getNode(i).getType()=="Bool")
+			{
+				if (nl.getNode(i).getValue()) 
+				{
+					out << "* ";
+				} 
+				else 
+				{
+					out << ". ";
+				}
 			}
 		}
-	} else {
-		for (int i = 0; i < nl.outputSize(); i++) {
-			if (nl.outputs[i].getNode().getValue()) {
+	} 
+	else 
+	{
+		for (int i = 0; i < nl.outputSize(); i++) 
+		{
+			if (nl.outputs[i].getNode().getValue()) 
+			{
 				out << "* ";
-			} else {
+			} 
+			else {
 				out << ". ";
 			}
 		}
@@ -1125,67 +1208,68 @@ void NodeList::visPrint(ostream &out, NodeList &nl)
 
 }
 
-	/**
-	 * Properly records bits for later use
-	 */
+/**
+ * Properly records bits for later use
+ */
 
 void NodeList::bitsPrint(NodeList &nl)
 {
 	//int j = nl.nodeSize();
 	for (int i = 0; i < nl.nodeSize(); i++) 
-		{
-			int pos = nl.inOutCalc( nl.getName(i));
-			if ( pos > -1 )   //If > -1 then Node found for avg calculations    
-			{ 
-				if (nl.getNode(i).getValue()) 
+	{
+		int pos = nl.inOutCalc(i);
+		if ( pos > -1 )   //If > -1 then Node found for avg calculations    
+		{ 
+			if (nl.getNode(i).getValue()) 
+			{
+				nl.bits[pos].push_back(true);
+				if(verbose)
 				{
-					nl.bits[pos].push_back(1);
-				       	if(verbose)
-					{
-						cerr<<"Printing 1 for Node "<<nl.getName(i)<<" to bit files"<<endl;
-						cerr<<"Value is "<<nl.getNode(i).getValue()<<endl;
-					}	
-				} 
-				else 
+					cerr<<"Printing 1 for Node "<<nl.getName(i)<<" to bit files"<<endl;
+					cerr<<"Value is "<<nl.getNode(i).getValue()<<endl;
+				}	
+			} 
+			else 
+			{
+				nl.bits[pos].push_back(false); 
+				if(verbose)
 				{
-					nl.bits[pos].push_back(0); 
-					if(verbose)
-					{
-						cerr<<"Printing 0 for Node "<<nl.getName(i)<<" to bit files"<<endl;
-						cerr<<"Value is "<<nl.getNode(i).getValue()<<endl;
-					}	
-				}
-
+					cerr<<"Printing 0 for Node "<<nl.getName(i)<<" to bit files"<<endl;
+					cerr<<"Value is "<<nl.getNode(i).getValue()<<endl;
+				}	
 			}
 
 		}
+
+	}
 	//j = int(nl.inputs.size());
 	for( int i = 0; i < int(nl.inputs.size()); ++i )
-		{
-			int pos = nl.getNodeIndex( nl.inputs[i].getName());
-			//cerr << "Pos: " << pos << endl;
-			if ( pos > -1 )   //If > -1 then Node found for avg calculations    
+	{
+		int pos = nl.getNodeIndex( nl.inputs[i].getName());
+		//cerr << "Pos: " << pos << endl;
+		if ( pos > -1 )   //If > -1 then Node found for avg calculations    
+		{/*
+			if (nl.getNode(pos).getValue()) 
 			{
-				if (nl.getNode(pos).getValue()) 
-				{
-					int val = 1;
-					nl.inputBits[i].push_back(val);
-					//cerr << "Pushing: one" << endl;
-				} 
-				else 
-				{
-					int val = 0;
-					nl.inputBits[i].push_back(val); 
-					//cerr << "Pushing: zero" << endl;
-				}
-			}
+				int val = 1;
+				nl.inputBits[i].push_back(val);
+				//cerr << "Pushing: one" << endl;
+			} 
+			else 
+			{
+				int val = 0;
+				nl.inputBits[i].push_back(val); 
+				//cerr << "Pushing: zero" << endl;
+			}*/
+			nl.inputBits[i].push_back(nl.getNode(pos).getValue());
 		}
+	}
 }
 
 
-	/**
-	 * Sorts Analysis points
-	 */
+/**
+ * Sorts Analysis points
+ */
 void NodeList::sortAnalPts()
 {
 	sort( analPts.begin(), analPts.end() );
@@ -1193,9 +1277,9 @@ void NodeList::sortAnalPts()
 
 }
 
-	/**
-	 * gets the next analysis point
-	 */
+/**
+ * gets the next analysis point
+ */
 int NodeList::getNextAnalPt()
 {
 	int val;
@@ -1210,97 +1294,83 @@ int NodeList::getNextAnalPt()
 	return val;
 }
 
-	/**
-	 * Initilizes the bit files
-	 */
+/**
+ * Initilizes the bit files
+ */
 void NodeList::initBits( bool val )
 {
-	vector<int> tmp;
-	if( val == true )
-		tmp.push_back(1);
-	else
-		tmp.push_back(0);
+	vector<bool> tmp;
+	tmp.push_back(val);
 	bits.push_back(tmp);
 }
 
-	/**
-	 * Initializes the input bit files
-	 */
+/**
+ * Initializes the input bit files
+ */
 void NodeList::initInputBits( )
 {
 	//init with 0's
 	bool val = false;
-	vector<int> tmp;
-	if( val == true )
-		tmp.push_back(1);
-	else
-		tmp.push_back(0);
+	vector<bool> tmp;
+	tmp.push_back(val);
 	inputBits.push_back(tmp);
 }
 
-	/**
-	 * Print all node names
-	 */
+/**
+ * Print all node names
+ */
 NodeList &NodeList::printNames(ostream &out)
 {
-	if (outputAll) {
-		for (int i = 0; i < nodeSize(); i++) {
-			out << "Column " << setw(5) << i << ": " << getName(i) << endl;
-		}
-
-		out << setw(8) << " ";
-		for (int i = 0; i < nodeSize(); i++) {
-			out << setw(2) << (i/100) % 10;
-		}
-		out << endl;
-
-		out << setw(8) << " ";
-		for (int i = 0; i < nodeSize(); i++) {
-			out << setw(2) << (i/10) % 10;
-		}
-		out << endl;
-
-		out << setw(8) << " ";
-		for (int i = 0; i < nodeSize(); i++) {
-			out << setw(2) << i % 10;
-		}
-		out << endl;
-	} else {
-		for (int i = 0; i < outputSize(); i++) {
-			out << "Column " << setw(5) << i << ": " << outputs[i].getName() << endl;
-		}
-
-		out << setw(8) << " ";
-		for (int i = 0; i < outputSize(); i++) {
-			out << setw(2) << (i/100) % 10;
-		}
-		out << endl;
-
-		out << setw(8) << " ";
-		for (int i = 0; i < outputSize(); i++) {
-			out << setw(2) << (i/10) % 10;
-		}
-		out << endl;
-
-		out << setw(8) << " ";
-		for (int i = 0; i < outputSize(); i++) {
-			out << setw(2) << i % 10;
-		}
-		out << endl;
+	char c[10] = "Output";
+	if (outputAll) 
+	{
+		strcpy(c,"Bool");
 	}
+	int j = 0;
+	for (int i = 0; i < nodeSize(); i++) 
+	{
+		if(getNode(i).getType()==c)
+		{
+			out << "Column " << setw(5) << j << ": " << getNodeItem(i).getName() << endl;
+			++j;
+		}
+	}
+
+	out << setw(8) << " ";
+	for (int i = 0; i < j; i++) 
+	{
+		out << setw(2) << (i/100) % 10;
+	}
+	out << endl;
+
+	out << setw(8) << " ";
+	for (int i = 0; i < j; i++) 
+	{
+		out << setw(2) << (i/10) % 10;
+	}
+	out << endl;
+
+	out << setw(8) << " ";
+	for (int i = 0; i < j; i++) 
+	{
+		out << setw(2) << i % 10;
+	}
+	out << endl;
 
 	return *this;
 }
 
-	/**
-	 * Comment to be written
-	 */
+/**
+ * Calculate the average activity level for nodes
+ * for the last n iterations where n is the value
+ * of transient.
+ */
 void NodeList::calcAvg( void )
 {
 	for( int i = 0; i < outCalcSize(); ++i )
 	{
 		for( int j = transient; j < runTime; ++j )
-			if( bits[i][j] == 1 )
+			if( bits[i][j] == true )
 			{
 				avg[i] = avg[i]+1;
 			}
@@ -1309,9 +1379,10 @@ void NodeList::calcAvg( void )
 
 }
 
-	/**
-	 * Comment to be written
-	 */
+/**
+ * Calculates the average activity level of nodes
+ * based on the average period
+ */
 void NodeList::calcCurrAvg( void )
 {
 	int tmp;
@@ -1323,7 +1394,7 @@ void NodeList::calcCurrAvg( void )
 			tmp = bits[i].size() - avgPeriod;
 
 		for( int j = tmp; j < int(bits[i].size()); ++j )
-			if( bits[i][j] == 1 )
+			if( bits[i][j] == true )
 			{
 				avg[i] = avg[i]+1;
 			}
@@ -1335,9 +1406,9 @@ void NodeList::calcCurrAvg( void )
 
 }
 
-	/**
-	 * Comment to be written
-	 */
+/**
+ * Prints the average activity levels to ostream out
+ */
 void NodeList::printAvg( ostream & out )
 {
 	float decAvg;	
@@ -1355,9 +1426,9 @@ void NodeList::printAvg( ostream & out )
 
 
 
-	/**
-	 * Prints nodes to ostream out
-	 */
+/**
+ * Prints nodes to ostream out
+ */
 void NodeList::printNodes( ostream & out)
 {
 	string nodeName;
@@ -1368,9 +1439,9 @@ void NodeList::printNodes( ostream & out)
 	}
 }
 
-	/**
-	 * Prints inputnodes to ostream out
-	 */
+/**
+ * Prints inputnodes to ostream out
+ */
 void NodeList::printInputs( ostream & out)
 {
 	string nodeName;
@@ -1381,9 +1452,9 @@ void NodeList::printInputs( ostream & out)
 	}
 }
 
-	/**
-	 * Comment to be written
-	 */
+/**
+ * Prints each node's average to an individual file in the directory path
+ */
 void NodeList::printAvgSep( const string & path )
 {
 	ofstream out;
@@ -1412,9 +1483,40 @@ void NodeList::printAvgSep( const string & path )
 	}
 }
 
-	/**
-	 * Comment to be written
-	 */
+/**
+ * returns the recent average activity for 'node'
+ * for the last n iterations where n is the 
+ * value of transient
+ */
+int NodeList::getNodeAvg(string node, int transient)
+{
+	int rval;
+	float total=0;
+	bool set=false;
+	if(transient>runTime)
+		transient=runTime;
+	for(int i=0; i<outCalcSize(); ++i)
+	{
+		if(node==outCalc[i].getName())
+		{
+			for(int j=runTime; j>(runTime-transient); --j)
+			{
+				if(bits[i][j]==true)
+					++total;
+			}
+			set=true;
+			rval=int(((total*100/transient)+0.5));
+		}
+	}
+	if(!set)
+		rval=-1;
+
+	return rval;
+}
+
+/**
+ * Prints the dosages to ostream out
+ */
 void NodeList::printDosages( ostream & out )
 {
 	//ofstream out;
@@ -1428,16 +1530,17 @@ void NodeList::printDosages( ostream & out )
 }
 
 
-	/**
-	 * Print bits by iterations  -> 1 line = state of the system
-	 */
+/**
+ * Print bits by iterations  -> 1 line = state of the system
+ */
 void NodeList::printBits( ostream & out )
 {
 	for( int j = 1; j < runTime; ++j )
 	{
 		for( int i = 0; i < outCalcSize(); ++i )
 		{
-			out << bits[i][j] << "  ";
+			char c = (bits[i][j]?'1':'0');
+			out << c << "  ";
 			//cerr << outCalc[i].getName() << ":  ";
 		}
 		out << endl;
@@ -1446,9 +1549,9 @@ void NodeList::printBits( ostream & out )
 	out << endl;
 }
 
-	/**
-	 * Prints input bits
-	 */
+/**
+ * Prints input node bits to ostream out
+ */
 void NodeList::printInputBits( ostream & out )
 { 
 	int numOfInputs = getNumOfInputs();
@@ -1456,7 +1559,7 @@ void NodeList::printInputBits( ostream & out )
 	{
 		for( int i = 0; i < numOfInputs; ++i )
 		{
-			out << inputBits[i][j] << "  ";
+			out << (inputBits[i][j]?'1':'0') << "  ";
 			//cerr << inputs[i].getName() << ":  ";
 		}
 		out << endl;
@@ -1465,3 +1568,77 @@ void NodeList::printInputBits( ostream & out )
 	out << endl;
 
 }
+
+string NodeList::getInputNames(int node)
+{
+	/*
+	 * getInputNames returns a string of input node
+	 * names in the order they are in the NodeList
+	 */
+	int j=0;
+	string arr = "";
+	for(int i=0; i<nodeSize(); ++i)
+	{
+		if(getNode(i).getType()=="Input")
+		{
+			if(j==node)
+			{
+				arr=getNodeItem(i).getName();
+				i=nodeSize();
+			}
+			++j;
+		}
+	}
+	return arr;
+}
+
+string NodeList::getOutNames(int i)
+{
+	/*
+	 * getOutNames returns a string of output node
+	 * names in the order they are in the NodeList
+	 */
+	return outCalc[i].getName();
+}
+
+void NodeList::setInputs(string node, int level)
+{
+	/*
+	 * setInputs sets a node's input
+	 */
+	for(int j=0; j<nodeSize(); ++j)
+	{
+		if(getNode(j).getType()=="Input"&&getNodeItem(j).getName()==node)
+		{
+			static_cast<InputNode &>(getNode(j)).setLevels(level, runTime);
+		}
+	}
+}
+
+int NodeList::getOutLevels(int j)
+{
+	/*
+	 * getOutLevels returns an array of integers
+	 * reflecting the output levels of the output nodes
+	 */
+	float fAvg=(avg[j]*100);
+	int i = int(fAvg+0.5);
+	return i;
+}
+
+bool NodeList::isBool(string s)
+{
+	/**
+	 * returns true if s is a boolean node in the network
+	 */
+	bool rval = false;
+	for(int i=0; i<nodeSize(); ++i)
+	{
+		if(getNode(i).getType()=="Bool"&&nodes[i].getName()==s)
+		{
+			rval = true;
+		}
+	}
+	return rval;
+
+}
