@@ -19,10 +19,10 @@
 #include <iostream>
 #include <fstream>
 #include <errno.h>
+#include <cstring>
 #include "NodeList.h"
 #include "Node.h"
 #include "lib/Matrix.h"
-#include "exts/FileConverter/Generator.h"
 #include "Simulation.h"
 
 using namespace std;
@@ -32,6 +32,9 @@ Simulation::Simulation(void)
 	/**
 	 * Simulation constructor that initializs variables
 	 */
+	srand(time(NULL));
+	noOutFiles=false;
+	usePlugins=true;
 	ROOTDIR = "../CCoutput";
 	SIMDIR = ROOTDIR+"/stats";
 	NODESAVGDIR = "NodesAvg";
@@ -49,6 +52,7 @@ Simulation::Simulation(void)
 	in_patternNodesFile = "";
 	iNodeList = "";
 	TT2ND = false;
+	rand_nodeInputs=false;
 	ND2TT = false;
 	patternAnal = false;
 	rand_init = false;
@@ -57,6 +61,7 @@ Simulation::Simulation(void)
 	iTables = "";
 	mutRate_nodes = 0;
 	mutRate_logic = 0;
+	rand_logicMut=false;
 	cycleSize = 0;
 	outputFile = "";
 	visualFile = "";
@@ -72,8 +77,9 @@ Simulation::~Simulation(void)
 	;;;
 }
 
-void Simulation::setup(int argc, char* argv[])
-{	/**
+void Simulation::setup(int argc, char** argv)
+{	
+	/**
 	 * Setup, takes in argv, its length and 
 	 * cycles through argv to initialize flags, 
 	 * it then creates apropriate directories 
@@ -81,7 +87,7 @@ void Simulation::setup(int argc, char* argv[])
 	 */
 	for (int i = 1; i < argc; i++) 
 	{
-		
+
 		if (strcmp(argv[i],"-inodelist") == 0)
 		{
 			i++;
@@ -92,6 +98,16 @@ void Simulation::setup(int argc, char* argv[])
 			i++;
 			noBits=true;
 		}
+		else if (strcmp(argv[i],"-noPrint") == 0)
+		{
+			i++;
+			noOutFiles=true;
+		}
+		else if (strcmp(argv[i],"-noPlugins") == 0)
+		{
+			i++;
+			usePlugins=false;
+		}
 		else if (strcmp(argv[i],"-itables") == 0)
 		{
 			i++;
@@ -110,7 +126,7 @@ void Simulation::setup(int argc, char* argv[])
 			i++;
 			outputFile = argv[i];
 		}
-		else if (strcmp(argv[i],"-h") == 0) 
+		else if (strcmp(argv[i],"-h") == 0||strcmp(argv[i],"-help")==0) 
 		{
 			print_help();
 			exit(1);
@@ -152,44 +168,17 @@ void Simulation::setup(int argc, char* argv[])
 		{
 			outputAll = true;
 		}
-		else if (strcmp(argv[i],"-patterns") == 0) 
+		else if (strcmp(argv[i],"-rInputs") == 0) 
 		{
-			patternAnal = true;
+			rand_nodeInputs = true;
 			i++;
-			if (strcmp(argv[i],"-isettings") == 0)
-			{ 
-				i++;
-				in_patternSpecsFile = argv[i];
-				i++;
-				if (strcmp(argv[i],"-inodes") == 0)
-				{ 
-					i++;
-					in_patternNodesFile = argv[i];
-				}
-				else 
-				{
-					cerr << "Invalid argument: " << argv[i] << endl;
-					print_help();
-					exit(1);
-				}
-			}
-			else if (strcmp(argv[i],"-inodes") == 0)
-			{ 
-				i++;
-				in_patternNodesFile = argv[i];
-				i++;
-				if (strcmp(argv[i],"-ispecs") == 0)
-				{ 
-					i++;
-					in_patternSpecsFile = argv[i];
-				}
-				else 
-				{
-					cerr << "Invalid argument: " << argv[i] << endl;
-					print_help();
-					exit(1);
-				}
-			}
+			mutRate_nodes = atoi(argv[i]);
+		}
+		else if (strcmp(argv[i],"-r_mut") == 0) 
+		{
+			rand_logicMut = true;
+			i++;
+			mutRate_logic = atoi(argv[i]);
 		} 
 		else if (strcmp(argv[i],"-randInit") == 0) 
 		{
@@ -206,17 +195,17 @@ void Simulation::setup(int argc, char* argv[])
 			exit(1);
 		}
 	} //end for 
-	
+
 	if (!visout) 
-  	{
+	{
 		cerr << "No output file given for visual mode" << endl;
 		exit(1);
 	}
 
 	cmd = "mkdir " + ROOTDIR + DUMP;
 	system(cmd.c_str());
-	
-	
+
+
 	calcModeSetup();
 	visModeSetup();
 
@@ -228,25 +217,25 @@ void Simulation::calcModeSetup(void)
 	/**
 	 * creates the appropriate directories for calc mode
 	 */
-	if(calc)
+	if(calc&&!noOutFiles)
 	{
-	cmd = "mkdir " +  SIMDIR + DUMP; 
-	system(cmd.c_str());
-	runDir = SIMDIR + "/" + runName;
-	cmd = "mkdir " +  runDir + DUMP; 
-	system(cmd.c_str());
-	cmd = "mkdir " + runDir + "/" + NODESAVGDIR + DUMP;  
-	system(cmd.c_str());
-	cmd = "mkdir " + runDir + "/" + BITSDIR + DUMP;  
-	system(cmd.c_str());
-	cmd = "mkdir " + runDir + "/" + LOGICDIR + DUMP;  
-	system(cmd.c_str());
-	inputFileCheck();
-	cmd = "cp " + in_specsFile + " " + SIMDIR + "/" + runName + "/specs.txt";  
-	system(cmd.c_str());	
-	cmd = "cp " + in_logicFile + " " + SIMDIR+"/"+runName+"/logic/logic.txt";
-	system(cmd.c_str());
-	avgFileName = SIMDIR + "/" + runName + "/" + "allNodes_avg" + ext;
+		cmd = "mkdir " +  SIMDIR + DUMP; 
+		system(cmd.c_str());
+		runDir = SIMDIR + "/" + runName;
+		cmd = "mkdir " +  runDir + DUMP; 
+		system(cmd.c_str());
+		cmd = "mkdir " + runDir + "/" + NODESAVGDIR + DUMP;  
+		system(cmd.c_str());
+		cmd = "mkdir " + runDir + "/" + BITSDIR + DUMP;  
+		system(cmd.c_str());
+		cmd = "mkdir " + runDir + "/" + LOGICDIR + DUMP;  
+		system(cmd.c_str());
+		inputFileCheck();
+		cmd = "cp " + in_specsFile + " " + SIMDIR + "/" + runName + "/specs.txt";  
+		system(cmd.c_str());	
+		cmd = "cp " + in_logicFile + " " + SIMDIR+"/"+runName+"/logic/logic.txt";
+		system(cmd.c_str());
+		avgFileName = SIMDIR + "/" + runName + "/" + "allNodes_avg" + ext;
 	}
 }
 
@@ -259,14 +248,14 @@ void Simulation::visModeSetup(void)
 	{
 		cmd = ROOTDIR+"/"+visualFile;
 		visout.open(cmd.c_str());
-			if (!visout) 
-			{
-				cerr << "Unable to open " << visualFile << " for input" << endl;
-				exit(1);
-			}
-	inputFileCheck();
+		if (!visout) 
+		{
+			cerr << "Unable to open " << visualFile << " for input" << endl;
+			exit(1);
+		}
+		inputFileCheck();
 	}
-	
+
 }
 
 void Simulation::inputFileCheck(void)
@@ -294,16 +283,26 @@ void Simulation::inputGenerator(void)
 	/**
 	 * Uses the generator class to create a logic or truth table file
 	 */
-	Generator g;
+	PluginWrapper p;
+	p.open("Plugins/FileConverter/FileConverter.so");
+	p.getPlugin()->setSIMDIR(SIMDIR);
+	p.getPlugin()->setRunName(runName);
+	p.getPlugin()->setDUMP(DUMP);
+	p.getPlugin()->setROOTDIR(ROOTDIR);
+	p.getPlugin()->setOutFile(outputFile);
+	p.getPlugin()->setup("Plugins");
+	p.getPlugin()->setLogic(in_logicFile);
+	p.getPlugin()->setSpecs(in_specsFile);
 	if(TT2ND)
 	{
-		g.set_tablesPath( iTables );
-        	g.cchain( iNodeList ,  ROOTDIR+"/"+outputFile, rand_init, false );
+		p.getPlugin()->midSim();
 	}
 	if(ND2TT)
 	{
-		g.cchainToTables( in_logicFile, ROOTDIR+"/"+outputFile);
+		p.getPlugin()->postSim();
 	}
+
+
 }
 
 void Simulation::run(void)
@@ -317,7 +316,7 @@ void Simulation::run(void)
 	if ((TT2ND==true)||(ND2TT==true))
 	{
 		inputGenerator();
-		
+
 	}
 	else if (analysisMode==true)
 	{
@@ -338,53 +337,62 @@ void Simulation::run(void)
 void Simulation::simulate(void)
 {
 
-	/*
+	if(usePlugins)
+	{
+		pManager.setROOTDIR(ROOTDIR);
+		pManager.setOutFile(outputFile);
+		pManager.setSIMDIR(SIMDIR);
+		pManager.setRunName(runName);
+		pManager.setDUMP(DUMP);
+		pManager.setLogic(in_logicFile);
+		pManager.setSpecs(in_specsFile);
+		pManager.loadPlugins("Plugins");
+	}
+	/**
 	 * This function represents the bulk of the program
 	 * It creates a Boolean Network in the form of a NodeList
 	 * and evaluates the network a set number of times,
 	 * making some analysis throughout.
 	 */
 
-	//*********************************** Pre-Simulation Operations**************************************//	
-	//Randomly generate inputs to each node while keeping logic
-	//and the number of inputs for each node the same.
 
-//*********************************End of Pre-Simulation Operations***********************************//	
-
-	if(rand_init)
+	//plugin manager initilization activities
+	if(usePlugins)
 	{
-		Generator g;
-		iTables=SIMDIR+"/"+runName+"/Tables";
-		system(("mkdir "+iTables+DUMP).c_str());
-		string path=iTables+"/nodesList.csv";
-		g.printNodesList(in_specsFile, in_logicFile, path);
-		g.cchainToTables( in_logicFile, iTables);
-
+		pManager.initialize();
+		if(pManager.fileChange())
+		{
+			in_specsFile=pManager.getSpecs();
+			in_logicFile=pManager.getLogic();
+		}
 	}
+
 	for( int run = 1; run <= n; ++run )
 	{
 		cerr << "Running Simulation: " << run << endl;
 		//Convert integer to string
 		char sim [10];
 		sprintf( sim,"%d", run );
-	
-	//********* Early Simulation Operations *********//
-	// Any operations that need to take place at the beginning of a new 
-	// simulation - i.e., generate new logic
-	
-		if( rand_init ) //Generate random initial conditions
+
+		//********* Early Simulation Operations *********//
+		// Any operations that need to take place at the beginning of a new 
+		// simulation - i.e., generate new logic
+
+		//plugin manager pre simulation activities
+		if(usePlugins)
 		{
-
-			Generator g;
-	
-			g.set_tablesPath( iTables );
-        		g.cchain( iTables+"/nodesList.csv", SIMDIR + "/" + runName + "/" + LOGICDIR + "/logic" + sim + ".txt", true, false );
-			in_logicFile = 	SIMDIR + "/" + runName + "/" + LOGICDIR + "/logic"+sim+".txt";
-			//in_logicFile = 	"../" + runName + "/Logic/logic" + sim + ".txt";
+			pManager.preSim();
+			if(pManager.fileChange())
+			{
+				in_specsFile=pManager.getSpecs();
+				in_logicFile=pManager.getLogic();
+			}
 		}
-	//********* End of Early Simulation Operations *********//
+
+		//********* End of Early Simulation Operations *********//
 
 
+		//open specs and logic files
 		ifstream in_specs, in_logic;
 		in_specs.open(in_specsFile.c_str());
 		if (in_specs.fail()) 
@@ -398,14 +406,20 @@ void Simulation::simulate(void)
 			cerr << "Unable to open " << in_logicFile << " for logic file" << endl;
 			exit(1);
 		}
+
+		//create nodelist
 		NodeList NList;
-		NList.setRunName( runName );;    
-		// Process Node descriptions
+		NList.setRunName( runName );  
+
+		// Process logic and specs files
 		if (verbose) cerr << "Node Descriptions" << endl;
 		in_logic >> NList;
 		in_specs >> NList;
 		in_logic.close();
 		in_specs.close();
+
+		//checks to make sure that every node named 
+		//as an input to another node exists
 		if(verbose) cerr<<"Checking Network Consistency"<<endl;
 		bool inconsistent=false;
 		for (int i = 0; i < NList.nodeSize(); i++) 
@@ -422,9 +436,11 @@ void Simulation::simulate(void)
 			cerr<<"Check your network specs"<<endl;
 			exit(1);
 		}
+
 		NList.setMutants();
-		if(visMode) NList.printNames(visout);
-		if(calc)
+
+		//sets up for calc mode anlysis'
+		if(calc && !noOutFiles)
 		{
 			string nodeLabelsFileName = SIMDIR + "/" + runName + "/node_labels.csv";
 			ofstream outNodeLabels;
@@ -454,49 +470,51 @@ void Simulation::simulate(void)
 			outNodeLabels.close();
 			outInputLabels.close();
 		}
+
+		NList.setOutputAll(outputAll);
+		if(visMode) NList.printNames(visout);
+
 		// Check for errors in descriptionsInput:IN1:TruInput:IN1:True:R:F:1:5-10:29,31:45-50:39,71:95-100:29e:R:F:1:5-10:29,31:45-50:39,71:95-100:29
 		// Execute Simulation
 		if (verbose) cerr << "----------- Begin Simulation ------------" << endl;
 		int t = 1;
-		
+
 		if( calc && ( NList.ptsSize() > 0 ) )
 		{
 			cmd = "mkdir " + SIMDIR + "/" + runName + "/SnapshotAnalysis"+DUMP;  
 			system(cmd.c_str());
 		}
 		int analPt = NList.getNextAnalPt();
+		NList.inOutInit();
+
+		//this is the main loop representing an iteration
 		while (t < NList.getRunTime()) 
 		{
 			if(visMode) visout << setw(8) << t << " ";
 			if (verbose) cerr << ">>>>> Time " << t << endl;
-			// Evaluate each of the output nodes
 			if (verbose) cerr << "Processing " << NList.outputSize() << " output nodes" << endl;
-			if (outputAll)
+
+			for (int i = 0; i < NList.outputSize(); i++) 
 			{
-				for (int i = 0; i < NList.nodeSize(); i++) 
+
+				if (verbose) 
+					cerr << "Output Node " << NList.outputs[i].getName() << endl;
+				bool result = NList.outputs[i].getNode().evaluate(NList, t);
+				if (verbose)
 				{
-					if (verbose) cerr<<"Evaluating Node "<<i<<endl;
-					NList.getNode(i).evaluate(NList, t);
-				}
-			}
-			else
-			{
-				for (int i = 0; i < NList.outputSize(); i++) 
-				{
-				
-					if (verbose) 
-						cerr << "Output Node " << NList.outputs[i].getName() << endl;
-					bool result = NList.outputs[i].getNode().evaluate(NList, t);
-					if (verbose)
-					{
-						cerr << "Node "<< NList.outputs[i].getName()<<" Evaluated to "<<result<<endl;
-					}
+					cerr << "Node "<< NList.outputs[i].getName()<<" Evaluated to "<<result<<endl;
 				}
 			}
+
 			if (visMode) NList.visPrint(visout, NList);
-			if (calc&&!noBits) NList.bitsPrint(NList);
-			
+			if (calc) NList.bitsPrint(NList);
+
 			ipause();
+			if(usePlugins)
+			{
+				pManager.setT(t);
+				pManager.setAnalysisPoint(analPt);
+			}
 			//Perform incremental output analysis during the simulation 
 			//based on the OutputAnalysis option
 			if( calc && (( NList.ptsSize() > 0 )  || (analPt > 0)) )
@@ -515,7 +533,7 @@ void Simulation::simulate(void)
 					if (!outAvg) 
 					{
 						cerr << "Unable to open " << avgFileName_pt 
-							 << " for Avg. input" << endl;
+							<< " for Avg. input" << endl;
 						exit(1);
 					}
 
@@ -528,59 +546,57 @@ void Simulation::simulate(void)
 
 					string name = SIMDIR+"/"+runName + "/SnapshotAnalysis/" + folder+"/NodesAvg/";
 					NList.printAvgSep(name);
-					
-					if( patternAnal )
-					{
-					string labelsFileName = SIMDIR + "/" + runName + "/node_labels.csv";
-					string specAvgFile = SIMDIR + "/" + runName + "/SnapshotAnalysis/" + folder + "/patternNodes_avg.csv";
-					analObj.midPatterns(avgFileName_pt, labelsFileName, specAvgFile, in_patternNodesFile);	
 
-					}
-					
+
 					analPt = NList.getNextAnalPt();
 				}
 			}
-			
+			if(usePlugins)
+				pManager.midSim(NList);
 
 			t += 1;
-		}//end for loop
-		if( calc )
+		}//end for loop representing simulation step
+
+		//post iteration calculation work
+		if( calc && !noOutFiles)
 		{
-			bitsFileName = SIMDIR + "/" + runName + "/" + BITSDIR + "/" + "network_bits" + sim + ext;
-			outBits.open(bitsFileName.c_str() );
-			if (!outBits) 
+			if(!noBits)
 			{
-				cerr << "Unable to open " << bitsFileName 
-					 << " for Bit output" << endl;
-				exit(1);
+				bitsFileName = SIMDIR + "/" + runName + "/" + BITSDIR + "/" + "network_bits" + sim + ext;
+				outBits.open(bitsFileName.c_str() );
+				if (!outBits) 
+				{
+					cerr << "Unable to open " << bitsFileName 
+						<< " for Bit output" << endl;
+					exit(1);
+				}
+
+				inputBitsFileName = SIMDIR + "/" + runName + "/" + BITSDIR + "/" + "input_bits" + sim + ext;
+				outInputBits.open( inputBitsFileName.c_str() );
+				if (!outInputBits) 
+				{
+					cerr << "Unable to open " << inputBitsFileName 
+						<< " for Bit output" << endl;
+					exit(1);
+				}
 			}
-			
-			inputBitsFileName = SIMDIR + "/" + runName + "/" + BITSDIR + "/" + "input_bits" + sim + ext;
-			outInputBits.open( inputBitsFileName.c_str() );
-			if (!outInputBits) 
-			{
-				cerr << "Unable to open " << inputBitsFileName 
-					 << " for Bit output" << endl;
-				exit(1);
-			}
-			
 			outAvg.open(avgFileName.c_str(), ios::app);
 			if (!outAvg) 
 			{
 				cerr << "Unable to open " << avgFileName 
-					 << " for Avg. input" << endl;
+					<< " for Avg. input" << endl;
 				exit(1);
 			}
-			
+
 			string dosageFileName = SIMDIR + "/" + runName + "/input_dosages" + ext;        
 			outDosage.open(dosageFileName.c_str(), ios::app);
 			if (!outDosage) 
 			{
 				cerr << "Unable to open " << dosageFileName 
-					 << " for dosage data output!" << endl;
+					<< " for dosage data output!" << endl;
 				exit(1);
 			}
-			
+
 			cerr << "	Calculating averages ... ";
 			NList.calcAvg();
 			cerr << "done" << endl;
@@ -600,52 +616,43 @@ void Simulation::simulate(void)
 			outDosage.close();
 			cerr << "done" << endl;
 
-			cerr << "	Printing output bit files ... ";
-			NList.printBits(outBits);
-			outBits.close();
-			cerr << "done" << endl;
-			cerr << "	Printing input bit files ... ";
-			NList.printInputBits(outInputBits);
-			outInputBits.close();
-			cerr << "done" << endl;
-			
-			
+			if(!noBits)
+			{
+				cerr << "	Printing output bit files ... ";
+				NList.printBits(outBits);
+				outBits.close();
+				cerr << "done" << endl;
+				cerr << "	Printing input bit files ... ";
+				NList.printInputBits(outInputBits);
+				outInputBits.close();
+				cerr << "done" << endl;
+			}
 
 
 
-	//********* Late Simulation Operations *********//
-	// Any operation/calculation at the end of each simulation
-	// i.e., to calculate stats in real time instead of waiting 
-	// till the end of the simmulation cycle
-		
-		if( patternAnal )
-		{
 
-			string labelsFileName = SIMDIR + "/" + runName + "/node_labels" + ext;
-			string specAvgFile = SIMDIR + "/" + runName + "/patternNodes_avg" + ext;
-			analObj.midPatterns(avgFileName, labelsFileName, specAvgFile, in_patternNodesFile);
+			//********* Late Simulation Operations *********//
+			// Any operation/calculation at the end of each simulation
+			// i.e., to calculate stats in real time instead of waiting 
+			// till the end of the simmulation cycle
 
-			string iSpecsFileName = in_patternSpecsFile;
-			string iNodesFileName = in_patternNodesFile;
+			if(usePlugins)
+			{
+				pManager.setT(t);
+				pManager.postSim();
+			}
 
-			string iInNodesFileName = SIMDIR + "/" + runName + "/input_labels" + ext;
-			string iAvgFileName = SIMDIR + "/" + runName + "/patternNodes_avg" + ext;
-			string iInAvgFileName = SIMDIR + "/" + runName + "/input_dosages" + ext;
-			string outPath = SIMDIR + "/" + runName + "/patterns";	
+			//********* End of Late Simulation Operations *********//
 
-			analObj.postPatterns(iSpecsFileName, iNodesFileName, iInNodesFileName, iAvgFileName, iInAvgFileName, outPath);
-			
-		}//end if( patternAnal )
-			
-	//********* End of Late Simulation Operations *********//
-			
 
 		} // end if( calc )
-		
+
 	} //end for loop 
-//***************************************** End of Simulation ****************************************//
+	//***************************************** End of Simulation ****************************************//
 	cout<<"Simulation runs completed!"<<endl;
-		
+
+	if(usePlugins)
+		pManager.close();
 }	
 
 
@@ -683,7 +690,7 @@ void Simulation::print_help()
 	string space = "	";
 	int width1 = 20;
 	int width2 = 45;
-	cout<<"Usage: ChemChain [OPTIONS]"<< endl;
+	cout<<"Usage: ChemChain(version 1.20) [OPTIONS]"<< endl;
 	cout<<endl;
 	cout<<endl<<"Modes:"<<endl;
 	cerr<<space<<left<<setw(width1)<<"-calc xxx"<<setw(width2)
@@ -715,15 +722,12 @@ void Simulation::print_help()
 	cerr<<space<<left<<setw(width1)<<"-A"<<setw(width2)
 		<<"Evaluates and prints all nodes in the network (used with -vis)"<<endl;
 	cerr<<space<<left<<setw(width1)<<"-noBits"<<setw(width2)
-		<<"Suppress bitfile printing"<<endl;
+		<<"Suppress calculcation of bitfiles"<<endl;
+	cerr<<space<<left<<setw(width1)<<"-noPrint"<<setw(width2)
+		<<"Suppress printing outputfiles"<<endl;
+	cerr<<space<<left<<setw(width1)<<"-noPlugins"<<setw(width2)
+		<<"Suppress running plugins"<<endl;
 	cerr<<space<<left<<setw(width1)<<"-randInit"<<setw(width2)
 		<<"Generate random network initial conditions for each simulation"<<endl;
-	
-	cout<<endl<<"Patterns Analysis:"<<endl;
-	cerr<<space<<left<<setw(width1)<<"-patterns"<<setw(width2)
-		<<"Instatiate patterns analysis (Requires -iSpecs and -iNodes flags)"<<endl;
-	cerr<<space<<left<<setw(width1)<<"-isettings xxx"<<setw(width2)
-		<<"Patterns file with node activity range settings"<<endl;	
-	cerr<<space<<left<<setw(width1)<<"-inodes xxx"<<setw(width2)
-		<<"File with nodes to be analyzed"<<endl;
+
 }
